--	Copyright (C) 2004 by neuron
--	aagaande AT gmail DOT com
--	
--	This program is free software; you can redistribute it and--or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation; either version 2 of the License, or
--	(at your option) any later version.
--	
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--	
--	You should have received a copy of the GNU General Public License
--	along with this program; if not, write to the
--	Free Software Foundation, Inc.,
--	59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

local nQuestLog = nQuestLog
local moduleName = "Notifications"
local Notification = nQuestLog:NewModule(moduleName,"AceEvent-2.0","AceHook-2.1", "AceConsole-2.0", "Sink-1.0")

local Quixote = AceLibrary("Quixote-1.0")

local L = AceLibrary("AceLocale-2.2"):new("Notifications")

function Notification:OnModuleLoad()
	self.menuName = L["Notifications"]
	self.description = L["Description"]
	
	self.core:RegisterDefaults(moduleName, "profile", {
		["enabled"] = true,
		["notifyFinishedQuest"] = true,
		["notifyFinishedQuestSound"] = true,
		["notifyFinishedObjective"] = false,
		["notifyLootedQuestItem"] = false,
		["notifyLootedHiddenQuestItemOnly"] = false,
	})
end

function Notification:ToggleEnabled(v)
	self.db.profile.enabled = v
	if (v) then
		self:RegisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
		self:RegisterEvent("Quixote_Quest_Complete", "OnQuest_Complete")
	else
		self:UnregisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
		self:UnregisterEvent("Quixote_Quest_Complete", "OnQuest_Complete")
	end
end

function Notification:GetMenu(oldMenu)
	local menu = {
		type="group",
		name=self.menuName,
		order=oldMenu.order,
		desc=self.description,
		args = {
			toggleEnabled = oldMenu,
			notifyFinishedQuest = {
				type="toggle",
				name=L["Finished quest"],
				desc=L["Notify on"] .. " " .. L["Finished quest"],
				order=1300,
				get=function()
					return self.db.profile.notifyFinishedQuest
				end,
				set=function(v)
					self.db.profile.notifyFinishedQuest = v
				end
			},
			notifyFinishedQuestSound = {
				type="toggle",
				name=" - " .. L["notifyFinishedQuestSound"],
				desc=L["notifyFinishedQuestSound desc"],
				order=1301,
				get=function()
					return self.db.profile.notifyFinishedQuestSound
				end,
				set=function(v)
					self.db.profile.notifyFinishedQuestSound = v
				end
			},
			notifyFinishedObjective = {
				type="toggle",
				name=L["Finished objective"],
				desc=L["Notify on"] .. " " .. L["Finished objective"],
				order=1302,
				get=function()
					return self.db.profile.notifyFinishedObjective
				end,
				set=function(v)
					self.db.profile.notifyFinishedObjective = v
				end
			},
			notifyFinishedQuestItem = {
				type="toggle",
				name=L["Getting quest item"],
				desc=L["Notify on"] .. " " .. L["Getting quest item"],
				order=1303,
				disabled=function()
					return self.db.profile.notifyLootedHiddenQuestItemOnly
				end,
				get=function()
					return self.db.profile.notifyLootedQuestItem
				end,
				set=function(v)
					self.db.profile.notifyLootedQuestItem = v
				end
			},
			notifyFinishedHiddenQuestItem = {
				type="toggle",
				name=" - " .. L["only when quest is hidden"],
				desc=L["Notify on"] .. " " .. L["Getting quest item when the quest is hidden"],
				order=1304,
				disabled=function()
					return not self.db.profile.notifyLootedQuestItem
				end,
				get=function()
					return self.db.profile.notifyLootedHiddenQuestItemOnly
				end,
				set=function(v)
					self.db.profile.notifyLootedHiddenQuestItemOnly = v
				end
			},
		}
	}
	
	-- Add SinkLib options menu
	menu.args.output = AceLibrary("Sink-1.0"):GetAceOptionsDataTable(self).output
	
	-- Set SinkLib menu order
	menu.args.output.order = 1299
	
	-- Add testing option and additional spacer line to SinkLib options menu
	menu.args.output.args.nQLoutputTest = {
		type = 'execute',
		name = L["nQLoutputTest"],
		desc = L["nQLoutputTest desc"],
		order = -101,
		func = function()
			self:Notify(L["fakeQuest"],true)
		end
	}
	menu.args.output.args.nQLtestSpacer = {
		type = 'header',
		name = ' ',
		order = -100,
	}
	
	return menu
end

function Notification:Notify(text,finished)
	if (finished and self.db.profile.notifyFinishedQuestSound) then
		PlaySoundFile("Sound\\Creature\\Peon\\PeonBuildingComplete1.wav")
	end
	
	local color = self.core:GetColor("notificationColor")
	self:Pour(text, color.r / 255, color.g / 255, color.b / 255)
end

function Notification:OnQuest_Leaderboard_Update(title, _, _, desc, had, got, needed)
	if (got == needed and self.db.profile.notifyFinishedObjective) then
		self:Notify(L["Finished objective"] .. " : " .. desc)
	elseif (got > 0) then
		local _, _, _, _, _, _, zone = Quixote:GetQuestByName(title)
		if (self.db.profile.notifyLootedQuestItem and not self.db.profile.notifyLootedHiddenQuestItemOnly) then
			-- Show alert, not locked to hidden only
			self:Notify(desc .. " " .. got .. "/"  .. needed)
		elseif ((self.db.profile.notifyLootedHiddenQuestItemOnly) and (not self.core.db.char.watchedQuests[title] or self.core.db.char.hiddenZones[zone])) then
			-- Show is hidden only, but quest or zone is hidden.
			self:Notify(desc .. " " .. got .. "/"  .. needed)
		end
	end
end

function Notification:OnQuest_Complete(title, _)
	if (self.db.profile.notifyFinishedQuest) then
		self:Notify(L["Finished quest"] .. " : " .. title,true)
	end
end